
//
// EncodingTypeEditor.cs
//
// Author:
//    Tomas Restrepo (tomasr@mvps.org)
//

using System;
using System.Collections;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing.Design;
using System.Text;
using System.Windows.Forms.Design;

namespace Winterdom.BizTalk.Samples.FixEncoding.Design
{
   /// <summary>
   /// Simple type editor for the Encoding property
   /// of our component, so that we can show a UI
   /// with the list of possible encodings 
   /// supported
   /// </summary>
   public class EncodingTypeEditor : UITypeEditor
   {
      private IWindowsFormsEditorService _editorService;
      private Encoding _selectedEncoding;
      private EncodingListControl _control;

      /// <summary>
      /// Make sure we tell the property grid we want a drop down 
      /// control
      /// </summary>
      /// <param name="context"></param>
      /// <returns></returns>
      public override UITypeEditorEditStyle GetEditStyle(ITypeDescriptorContext context)
      {
         System.Diagnostics.Debug.WriteLine("GetEditStyle: " + context);
         if ( context == null || context.Instance == null )
            return base.GetEditStyle(context);
         return UITypeEditorEditStyle.DropDown;
      }

      /// <summary>
      /// Popup the drop down with the encoding list
      /// </summary>
      /// <param name="context"></param>
      /// <param name="provider"></param>
      /// <param name="value"></param>
      /// <returns></returns>
      public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
      {
         System.Diagnostics.Debug.WriteLine("EditValue: " + context);
         if ( context == null || context.Instance == null || provider == null )
            return value;

         try
         {
            // get the editor service, just like in windows forms
            _editorService = (IWindowsFormsEditorService)
               provider.GetService(typeof(IWindowsFormsEditorService));

            _control = new EncodingListControl();
            _selectedEncoding = (Encoding)value;
            _control.SelectedCodePage = _selectedEncoding.CodePage;
            _control.Click += new EventHandler(OnControlClick);

            _editorService.DropDownControl(_control);
            return _selectedEncoding;
         }  
         finally
         {
            _editorService = null;
         } 
      }

      private void OnControlClick(object sender, EventArgs e)
      {
         _selectedEncoding = Encoding.GetEncoding(_control.SelectedCodePage);
         _editorService.CloseDropDown();
      }

   } // class EncodingTypeEditor

} // namespace Winterdom.BizTalk.Samples.FixEncoding.Design
